@@ -1,7 +1,21 @@
 // Swarm library. Copyright  1996-2000 Swarm Development Group.
-// This library is distributed without any warranty; without even the
-// implied warranty of merchantability or fitness for a particular purpose.
-// See file LICENSE for details and terms of copying.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation; either version 2 of the License, or
+// (at your option) any later version.
+//
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+// General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with this program; if not, write to the Free Software
+// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+// USA
+// 
+// The Swarm Development Group can be reached via our website at:
+// http://www.swarm.org/
 
 /*
 Name:         defobj.h
@@ -9,6 +23,60 @@ Description:  standard objects for GNU Objective C extensions
 Library:      defobj
 */
 
+#ifndef __swarm_defobj_h
+#define __swarm_defobj_h
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+typedef enum {fcall_type_void = 0,
+              fcall_type_boolean,
+              fcall_type_uchar, fcall_type_schar,
+              fcall_type_ushort, fcall_type_sshort,
+              fcall_type_uint, fcall_type_sint,
+              fcall_type_ulong, fcall_type_slong,
+              fcall_type_ulonglong, fcall_type_slonglong, 
+              fcall_type_float, fcall_type_double, fcall_type_long_double,
+              fcall_type_object, fcall_type_class,
+              fcall_type_string, fcall_type_selector, 
+              fcall_type_jobject, fcall_type_jstring, fcall_type_jselector,
+              fcall_type_iid
+              } fcall_type_t;
+
+#include <objc/objc.h>
+
+typedef union {
+  id object;
+  SEL selector;
+  Class _class;
+  const char *string;
+  BOOL boolean;
+  char schar;
+  unsigned char uchar;
+  short sshort;
+  unsigned short ushort;
+  int sint;
+  unsigned int uint;
+  long slong;
+  unsigned long ulong;
+  long long slonglong;
+  unsigned long long ulonglong;
+  float _float;
+  double _double;
+  long double _long_double;
+  void *iid;
+} types_t;
+
+typedef struct {
+  fcall_type_t type;
+  types_t val;
+} val_t;
+
+#define FCALL_TYPE_COUNT 23
+
+#ifdef __cplusplus
+}
+#else
 #import <defobj/deftype.h>
 #include <externvar.h>
 
@@ -20,7 +88,23 @@ Library:      defobj
 //D: creating objects and for storage allocation, error handling, and debugging
 //D: support.
 
-@protocol DefinedObject
+@protocol GetName
+//S: Get name which identifies object in its context of use.
+
+//D: Get name which identifies object in its context of use.
+
+//M: The getName message returns a null-terminated character string that
+//M: identifies an object in some context of use.  This message is commonly
+//M: used for objects that are created once in some fixed context where
+//M: they are also assigned a unique name.  Constant objects defined as
+//M: part of a program or library are examples.  This message is intended
+//M: only for returning a name associated with an object throughout its
+//M: lifetime.  It does not return any data that ever changes.
+USING
+- (const char *)getName;
+@end
+
+@protocol DefinedObject <GetName>
 //S: Object with defined type and implementation.
 
 //D: DefinedObject is the top-level supertype for all objects that follow
@@ -47,6 +131,7 @@ USING
 - (BOOL)respondsTo: (SEL)aSel;
 
 + (BOOL)conformsTo: (Protocol *)protocol;
+- (BOOL)conformsTo: (Protocol *)protocol;
 
 //M: getClass returns the class that implements the current behavior of an
 //M: object.
@@ -393,8 +478,6 @@ CREATING
 - (void)setPageSize: (size_t)pageSize;
 
 USING
-- getReclaimPolicy;
-- (BOOL)getStackedSubzones;
 - (size_t)getPageSize;
 
 //M: allocIVars: allocates the instance variable structure for a new
@@ -482,23 +565,7 @@ USING
 - (void)describeForEachID: outputCharStream;
 @end
 
-@protocol GetName
-//S: Get name which identifies object in its context of use.
-
-//D: Get name which identifies object in its context of use.
-
-//M: The getName message returns a null-terminated character string that
-//M: identifies an object in some context of use.  This message is commonly
-//M: used for objects that are created once in some fixed context where
-//M: they are also assigned a unique name.  Constant objects defined as
-//M: part of a program or library are examples.  This message is intended
-//M: only for returning a name associated with an object throughout its
-//M: lifetime.  It does not return any data that ever changes.
-USING
-- (const char *)getName;
-@end
-
-@protocol DefinedClass <DefinedObject, GetName>
+@protocol DefinedClass <DefinedObject>
 //S: Class which implements an interface of a type.
 
 //D: Class which implements an interface of a type.
@@ -525,9 +592,9 @@ CREATING
 - at: (SEL)aSel addMethod: (IMP)aMethod;
 - lispInCreate: expr;
 - hdf5InCreate: hdf5Obj;
-- lispOutShallow: stream;
-- hdf5OutShallow: hdf5Obj;
-- updateArchiver: archiver;
+- (void)lispOutShallow: stream;
+- (void)hdf5OutShallow: hdf5Obj;
+- (void)updateArchiver: archiver;
 USING
 - getDefiningClass;
 @end
@@ -676,7 +743,7 @@ USING
 //#: macro to raise Warning or Error with source location strings
 #define raiseEvent( eventType, formatString, args... ) \
 [eventType raiseEvent: \
-"\r" __FUNCTION__, __FILE__, __LINE__, formatString , ## args]
+"\r", __FUNCTION__, __FILE__, __LINE__, formatString , ## args]
 @end
 
 
@@ -698,7 +765,8 @@ externvar id <Warning>
   LibraryUsage,           //G: invalid usage of library interface
   DefaultAssumed,         //G: non-silent use of default
   ObsoleteFeature,        //G: using feature which could be removed in future
-  ObsoleteMessage;        //G: using message which could be removed in future
+  ObsoleteMessage,        //G: using message which could be removed in future
+  SaveWarning;            //G: non-fatal problem saving a resource  
 
 //#: macro used to create and initialize an Error symbol
 #define defwarning(name, message) \
@@ -741,13 +809,14 @@ externvar id <Error>
 //D: A class that provides customizable command line argument parsing support
 
 //E: Let's say you want to add a new argument, say `protocol' to your standard 
-//E: list of command.  In other words you want the following to happen at the
+//E: list of commands.  In other words you want the following to happen at the
 //E: command line when you type --help.
 //E: ------------------------
 //E: mgd@wijiji[/opt/src/mgd/src/mySwarmApp] $ ./mySwarmApp --help
 //E: Usage: mySwarmApp [OPTION...]
 //E: 
-//E:   -s, --varyseed             Run with a random seed
+//E:   -s, --varyseed             Select random number seed from current time
+//E:   -S, --seed=INTEGER         Specify seed for random numbers
 //E:   -b, --batch                Run in batch mode
 //E:   -m, --mode=MODE            Specify mode of use (for archiving)
 //E:   -p, --protocol=PROTOCOL    Set protocol
@@ -819,9 +888,7 @@ externvar id <Error>
 //E: }
 
 CREATING
-+ createBegin: (id <Zone>)aZone;
-- createEnd;
-- setArgc: (int)theArgc Argv: (const char **)theArgv;
+- setArgc: (unsigned)count Argv: (const char **)theArgv;
 - setAppName: (const char *)appName;
 - setAppModeString: (const char *)appModeString;
 - setOptionFunc: (int (*) (int, const char *))optionFunc;
@@ -829,8 +896,6 @@ CREATING
 - setVersion: (const char *)version;
 - addOptions: (struct argp_option *)options;
 
-+ createArgc: (int)argc Argv: (const char **)argv appName: (const char *)appName version: (const char *)version bugAddress: (const char *)bugAddress options: (struct argp_option *)options optionFunc: (int (*) (int, const char *))optionFunc inhibitExecutableSearchFlag: (BOOL)inhibitExecutableSearchFlag;
-
 //M: Takes an option specification that includes the following information:
 
 //M: - The name of the option specification
@@ -849,15 +914,18 @@ CREATING
 
 //M: - A sorting integer; relative placement of the option in the help
 //M:   screen.
+- (void)addOption: (const char *)name key: (int)key arg: (const char *)arg flags: (int)flags doc: (const char *)doc group: (int)group;
+
++ createArgc: (int)argc Argv: (const char **)argv appName: (const char *)appName version: (const char *)version bugAddress: (const char *)bugAddress options: (struct argp_option *)options optionFunc: (int (*) (int, const char *))optionFunc inhibitExecutableSearchFlag: (BOOL)inhibitExecutableSearchFlag;
 
 //M: This method is called for each option that occurs.
 - (int)parseKey: (int)key arg: (const char *)arg;
-
 SETTING
 - setInhibitArchiverLoadFlag: (BOOL)inhibitArchiverLoadFlag;
-- setInhibitExecutableSearch: (BOOL)theInhibitExecutableSearchFlag;
+- setInhibitExecutableSearchFlag: (BOOL)theInhibitExecutableSearchFlag;
 - setBatchModeFlag: (BOOL)batchModeFlag;
 - setVarySeedFlag: (BOOL)varySeedFlag;
+- setVerboseFlag: (BOOL)verboseFlag;
 //M: Specify a default path to use for configuration files when
 //M: installed location of Swarm cannot be determined. 
 //M: Defaults to current directory.
@@ -865,13 +933,19 @@ SETTING
 //M: Specify a default path to use for data files when installed location
 //M: of Swarm cannot be determined.  Defaults to current directory.
 - setDefaultAppDataPath: (const char *)path;
+- setFixedSeed: (unsigned)seed;
+
 
 USING
 - (BOOL)getBatchModeFlag;
 - (BOOL)getVarySeedFlag;
+- (BOOL)getFixedSeedFlag;
+- (unsigned)getFixedSeed;
+- (BOOL)getVerboseFlag;
 - (const char *)getAppName;
 - (const char *)getAppModeString;
 - (int)getArgc;
+- (int)getLastArgIndex;
 - (const char **)getArgv;
 - (const char *)getExecutablePath;
 - (const char *)getSwarmHome;
@@ -884,9 +958,10 @@ USING
 - (const char *)getAppConfigPath;
 - (BOOL)getShowCurrentTimeFlag;
 - (BOOL)getInhibitArchiverLoadFlag;
+//M: The user assigned seed value for random number generation
 @end
 
-@protocol Archiver <Create, Drop>
+@protocol Archiver <Create, Drop, RETURNABLE>
 //S: High level abstract serialization interface.
 //D: High level abstract serialization interface.
 CREATING
@@ -958,30 +1033,17 @@ SETTING
 USING
 @end
 
-typedef enum {fcall_type_void = 0,
-              fcall_type_boolean,
-              fcall_type_uchar, fcall_type_schar,
-              fcall_type_ushort, fcall_type_sshort, fcall_type_uint,
-              fcall_type_sint, fcall_type_ulong, fcall_type_slong,
-              fcall_type_slonglong, fcall_type_ulonglong,
-              fcall_type_float, fcall_type_double, 
-              fcall_type_long_double,
-              fcall_type_object, fcall_type_class,
-              fcall_type_string, fcall_type_selector, 
-              fcall_type_jobject, fcall_type_jstring,
-              } fcall_type_t;
-
 @protocol HDF5 <Create, Drop, CREATABLE>
 //S: HDF5 interface
 //D: HDF5 interface
 CREATING
-+ createBegin: (id <Zone>)aZone;
 - setWriteFlag: (BOOL)writeFlag;
 - setDatasetFlag: (BOOL)datasetFlag;
+- setExtensibleVectorType: (fcall_type_t)type;
+- setExtensibleDoubleVector;
 - setParent: parent;
 - setCompoundType: compoundType;
 - setCount: (unsigned)count;
-- createEnd;
 SETTING
 //M: Create-time use is to name the file or group.
 //M: Setting-time use is to rename component datasets that don't
@@ -995,8 +1057,9 @@ USING
 - (BOOL)getWriteFlag;
 - (size_t)getDatasetRank;
 - (size_t)getDatasetDimension: (unsigned)dimNumber;
-- (const char *)getName;
+- (fcall_type_t)getDatasetType;
 - (unsigned)getCount;
+- (const char *)getHDF5Name;
 - getCompoundType;
 - getClass;
 - (void)assignIvar: obj;
@@ -1006,6 +1069,9 @@ USING
 
 - (void)loadDataset: (void *)ptr;
 - (void)storeAsDataset: (const char *)name typeName: (const char *)typeName type: (fcall_type_t)type rank: (unsigned)rank dims: (unsigned *)dims ptr: (void *)ptr;
+
+- (void)addDoubleToVector: (double)val;
+
 - (void)storeTypeName: (const char *)typeName;
 - (void)storeComponentTypeName: (const char *)typeName;
 - (void)shallowLoadObject: obj;
@@ -1021,53 +1087,32 @@ USING
 - (void)storeAttribute: (const char *)attributeName value: (const char *)valueString;
 - (void)iterateAttributes: (int (*) (const char *key, const char *value))iterateFunc;
 - (const char *)getAttribute: (const char *)attributeName;
-
-- (void)drop;
+- (void)flush;
 @end
 
 @protocol HDF5CompoundType <Create, Drop, CREATABLE>
 //S: HDF5 composite type interface
 //D: HDF5 composite type interface
 CREATING
-+ createBegin: (id <Zone>)aZone;
 - setPrototype: prototype;
-- createEnd;
 USING
 - getPrototype;
 @end
 
-typedef union {
-  id object;
-  SEL selector;
-  Class class;
-  const char *string;
-  BOOL boolean;
-  char schar;
-  unsigned char uchar;
-  short sshort;
-  unsigned short ushort;
-  int sint;
-  unsigned int uint;
-  long slong;
-  unsigned long ulong;
-  long long slonglong;
-  unsigned long long ulonglong;
-  float _float;
-  double _double;
-  double _long_double;
-} types_t;
-
 typedef void *JOBJECT;
+typedef void *COMOBJECT;
 
 @protocol FArguments <Create, Drop, CREATABLE>
 //S: A language independent interface to dynamic call argument construction.
 //D: A language independent interface to dynamic call argument construction.
 CREATING
-+ createBegin: (id <Zone>)aZone;
-- setJavaFlag: (BOOL)javaFlag;
+- setLanguage: (id <Symbol>)languageType;
+//M: The selector is used to set argument types.  Some languages won't
+//M: have any, and so for those languages this need not be called.
 - setSelector: (SEL)aSel;
 - setJavaSignature: (const char *)javaSignature;
-+ create: (id <Zone>)aZone setSelector: (SEL)aSel setJavaFlag: (BOOL)javaFlag;
++ create: (id <Zone>)aZone setSelector: (SEL)aSel;
+- addArgument: (types_t *)value ofType: (fcall_type_t)type;
 - addArgument: (void *)value ofObjCType: (char)type;
 - addChar: (char)value;
 - addBoolean: (BOOL)value;
@@ -1085,31 +1130,35 @@ CREATING
 - addLongDouble: (long double)value;
 - addString: (const char *)value;
 - addObject: obj;
+- addSelector: (SEL)aSel;
 - addJavaObject: (JOBJECT)obj;
 - setObjCReturnType: (char)type;
+- setReturnType: (fcall_type_t)retType;
 - setBooleanReturnType;
-- createEnd;
 USING
-- (BOOL)getJavaFlag;
+- (val_t)getRetVal;
+- (id <Symbol>)getLanguage;
 - (void *)getResult;
 @end
 
-typedef enum callTypes { ccall, objccall, javacall, javastaticcall} call_t;
+typedef enum callTypes { ccall, COMcall, JScall, javacall, javastaticcall,  objccall } call_t;
 
 @protocol FCall <Create, Drop, CREATABLE>
 //S: A language independent interface to dynamic calls.
 //D: A language independent interface to dynamic calls.
 CREATING
-+ createBegin: (id <Zone>)aZone;
-- createEnd;
 + create: (id <Zone>)aZone target: obj
-                         selector: (SEL)aSel
+                         selector: (SEL)sel
+                        arguments: (id <FArguments>)fa;
++ create: (id <Zone>)aZone target: obj
+                       methodName: (const char *)methodName
                         arguments: (id <FArguments>)fa;
 - setArguments: args;
 - setFunctionPointer: (func_t)fn;
-- setMethod: (SEL)method inObject: object;
-- setJavaMethod: (const char *)methodName inObject: (JOBJECT)obj;
-- setJavaMethod: (const char *)methodName inClass: (const char *)className;
+- setMethodFromSelector: (SEL)method inObject: object;
+- setMethodFromName: (const char *)methodName inObject: object;
+- setJavaMethodFromName: (const char *)methodName inObject: (JOBJECT)jObj;
+- setJavaMethodFromName: (const char *)methodName inClass: (const char *)className;
 USING
 - (call_t)getCallType;
 - getArguments;
@@ -1150,22 +1199,177 @@ SETTING
 
 USING
 //M: Output a shallow Lisp representation of object state to a stream.
-- lispOutShallow: stream;
+- (void)lispOutShallow: stream;
 
 //M: Output a deep Lisp representation of object state to a stream.
-- lispOutDeep: stream;
+- (void)lispOutDeep: stream;
 
 //M: Output just key/variable pairs, where variables are serialized 
 //M: deep or shallow per deepFlag.
-- lispOutVars: stream deep: (BOOL)deepFlag;
+- (void)lispOutVars: stream deep: (BOOL)deepFlag;
+
+//M: On the given stream, save a Boolean variable called "aName" 
+//M: which has value "val".
+//M: Explanation: 
+//M: The Swarm lisp serialization approach assumes that objects have
+//M: lispOutDeep: and lispOutShallow: methods which indicate which
+//M: variables are supposed to be saved.  If an object is 
+//M: subclassed from SwarmObject, there are default lispOutDeep: 
+//M: and lispOutShallow: methods. Those methods employ on the method,
+//M: lispOutVars:deep:, which is the "default" approach to try to save
+//M: all variables, either deep or shallow.  Sometimes one needs
+//M: to selectively list particular instance variables to be saved.
+
+//M: This is necessary, for example, if one wants to save a Swarm
+//M: itself, because the usage of lispOutVars: will result in 
+//M: a variable "activity" being saved as nil, and so when the
+//M: saved values are read back in, the "activity" variable will be
+//M: erased and nil will appear in its place.  
+
+//M: Here is an example of how a subclass called "BFagent" 
+//M: might override lispOutDeep: to customize 
+//M: the selection of variables to be saved.  Note that the
+//M: same could be used to override lispOutShallow:. 
+
+//M: The key thing
+//M: to remember is that when one tries to do a deep save on
+//M: a high level object, such as a Swarm, then the Swarm
+//M: libraries will try to track from top to bottom, finding
+//M: all collections and objects, and all objects and collections
+//M: inside them, and so forth, and each will be told to execute
+//M: its lispOutDeep: method.  So all objects you want to save need
+//M: a lispOutDeep: method, or else the default will try to save all
+//M: variables.  If you omit some objects or variables from your
+//M: lispOutDeep: method, then they will not appear in the saved
+//M: file, which is what you want if you want to be sure that
+//M: pre-existing interited values of variables are not obliterated 
+//M: by bogus saved values.
+
+//E: - (void)lispOutDeep: stream
+//E: {
+//E:  [stream catStartMakeInstance: "BFagent"];
+//E:  [self lispSaveStream: stream Double: "demand" Value: demand];
+//E:  [self lispSaveStream: stream Double: "profit" Value: profit];
+//E:  [self lispSaveStream: stream Double: "wealth" Value: wealth];
+//E:  [self lispSaveStream: stream Double: "position" Value: position];
+//E:  [self lispSaveStream: stream Double: "cash" Value: cash];
+//E:  [self lispSaveStream: stream Double: "price" Value: price];
+//E:  [self lispSaveStream: stream Double: "dividend" Value: dividend];
+//E:  [self lispSaveStream: stream Integer: "myID" Value: myID];
+//E:  [stream catEndMakeInstance];
+//E:
+//E: }
+//E: An example of such a usage can be found in version 2.4 of the
+//E: Artificial Stock Market (http://ArtStkMkt.sourceforge.net). 
+- (void)lispSaveStream: stream Boolean: (const char *)aName Value: (int)val;
+
+//M: On the given stream, save a character variable called "aName"
+//M: which has value "val".
+- (void)lispSaveStream: stream Char: (const char *)aName Value: (char)val;
+
+//M: On the given stream, save a short integer variable called "aName"
+//M: which has value "val".
+- (void)lispSaveStream: stream Short: (const char *)aName Value: (short)val;
+
+//M: On the given stream, save an unsigned short integer variable
+//M: called "aName" which has value "val".
+- (void)lispSaveStream: stream UnsignedShort: (const char *)aName Value: (unsigned short)val;
+
+//M: On the given stream, save an integer variable called "aName"
+//M: which has value "val".
+- (void)lispSaveStream: stream Integer: (const char *)aName Value: (int)val;
+
+//M: On the given stream, save an unsigned integer variable called
+//M: "aName" which has value "val".
+- (void)lispSaveStream: stream Unsigned: (const char *) aName Value: (unsigned)val;
+
+//M: On the given stream, save a long variables called "aName"
+//M: which has value "val".
+- (void)lispSaveStream: stream Long: (const char *)aName Value: (long)val;
+
+//M: On the given stream, save an unsigned long variable
+//M: called "aName" which has value "val".
+- (void)lispSaveStream: stream UnsignedLong: (const char *)aName Value: (unsigned long)val;
+
+//M: On the given stream, save a long long variable called
+//M: "aName" which has value "val".
+- (void)lispSaveStream: stream LongLong: (const char *)aName Value: (long long)val;
+
+//M: On the given stream, save an unsigned long long variable called
+//M: "aName" which has value "val".
+- (void)lispSaveStream: stream UnsignedLongLong: (const char *) aName Value: (unsigned long long)val;
+
+//M: On the given stream, save a float valued variable called "aName"
+//M: which has value "val".
+- (void)lispSaveStream: stream Float: (const char *) aName Value: (double)val;
+
+//M: On the given stream, save a double valued variable called "aName"
+//M: which has value "val".
+- (void)lispSaveStream: stream Double: (const char *) aName Value: (double)val;
+
+
+//M: For customized archiving of dynamically allocated arrays within
+//M: objects. To use this method, override the object's lispOutDeep:
+//M: method as follows. The array is assumed allocated in one long
+//M: piece of memory, but it can be treated in segments to make it
+//M: two dimensional.  The number of rows is "rank" and the length of
+//M: the i'th row is dims[i], thus allowing for a ragged array.
+//M: 
+//M: In this example, Attribute is a class, culture is a
+//M: dynamically allocated array of numDims integers, so rank is 1 and
+//M: a pointer to numDims is passed through for the length of that row.
+
+//E: - (void)lispOutDeep: stream
+//E: {
+//E:  [stream catStartMakeInstance: "Attribute"];
+//E:  [super lispOutVars: stream deep: NO]; //saves all ints, doubles, BOOLs, and static arrays. Saves the values of all other objects as nil
+//E:  // Note one can use the previous "lispSaveStream:..." methods to
+//E:  // customize the choice of variables to be saved.
+//E:  // Now save an array called "culture", which has 1 row 
+//E:  // and "numDims" columns, onto the stream.
+//E:  [super lispStoreIntegerArray: culture Keyword: "culture" Rank: 1 Dims: &numDims Stream: stream];
+//E:  [stream catEndMakeInstance];
+//E: }
+- (void)lispStoreIntegerArray: (int *)ptr Keyword: (const char *)keyword Rank: (unsigned)rank Dims: (unsigned *)dims Stream: stream;
+
+//M: Lisp save array of Booleans, see lispStoreIntegerArray:Keyword:Rank:Dims.
+- (void)lispStoreBooleanArray: (BOOL *)ptr Keyword: (const char *)keyword Rank: (unsigned)rank Dims: (unsigned *)dims Stream: stream;
+
+//M: Lisp save array of characters, see lispStoreIntegerArray:Keyword:Rank:Dims.
+- (void)lispStoreCharArray: (char *)ptr Keyword: (const char *)keyword Rank: (unsigned)rank Dims: (unsigned *)dims Stream: stream;
+
+//M: Lisp save array of short integers, see lispStoreIntegerArray:Keyword:Rank:Dims.
+- (void)lispStoreShortArray: (short int *)ptr Keyword: (const char *)keyword Rank: (unsigned)rank Dims: (unsigned *)dims Stream: stream;
+
+//M: Lisp save array of unsigned integers, see lispStoreIntegerArray:Keyword:Rank:Dims.
+- (void)lispStoreUnsignedArray: (unsigned *)ptr Keyword: (const char *)keyword Rank: (unsigned)rank Dims: (unsigned *)dims Stream: stream;
+
+//M: Lisp save array of long, see lispStoreIntegerArray:Keyword:Rank:Dims.
+- (void)lispStoreLongArray: (long *)ptr Keyword: (const char *)keyword Rank: (unsigned)rank Dims: (unsigned *)dims Stream: stream;
+
+//M: Lisp save array of unsigned long, see lispStoreIntegerArray:Keyword:Rank:Dims.
+- (void)lispStoreUnsignedLongArray: (unsigned long *)ptr Keyword: (const char *)keyword Rank: (unsigned)rank Dims: (unsigned *)dims Stream: stream;
+
+//M: Lisp save array of long long, see lispStoreIntegerArray:Keyword:Rank:Dims.
+- (void)lispStoreLongLongArray: (long long *)ptr Keyword: (const char *)keyword Rank: (unsigned)rank Dims: (unsigned *)dims Stream: stream;
+
+//M: Lisp save array of unsigned long long, see lispStoreIntegerArray:Keyword:Rank:Dims.
+- (void)lispStoreUnsignedLongLongArray: (unsigned long long *)ptr Keyword: (const char *)keyword Rank: (unsigned)rank Dims: (unsigned *)dims Stream: stream;
+
+//M: Lisp save array of floats, see lispStoreIntegerArray:Keyword:Rank:Dims.
+- (void)lispStoreFloatArray: (float *)ptr Keyword: (const char *)keyword Rank: (unsigned)rank Dims: (unsigned *)dims Stream: stream;
+
+//M: Lisp save array of doubles, see lispStoreIntegerArray:Keyword:Rank:Dims.
+- (void)lispStoreDoubleArray: (double *)ptr Keyword: (const char *)keyword Rank: (unsigned)rank Dims: (unsigned *)dims Stream: stream;
+
 
 //M: Output a shallow HDF5 representation of object state to a stream.
-- hdf5OutShallow: (id <HDF5>)hdf5obj;
+- (void)hdf5OutShallow: (id <HDF5>)hdf5obj;
 
 //M: Output a deep HDF5 representation of object state to a stream.
-- hdf5OutDeep: (id <HDF5>)hdf5obj;
+- (void)hdf5OutDeep: (id <HDF5>)hdf5obj;
 
-- updateArchiver: archiver;
+- (void)updateArchiver: archiver;
 
 //F: Expect and convert a boolean from next index item.
 extern BOOL lispInBoolean (id index);
@@ -1182,7 +1386,7 @@ extern id lispInKeyword (id index);
 @end
 
 //G: The singleton Arguments object.
-externvar id arguments;
+externvar id <Arguments> arguments;
 
 //G: The singleton HDF5 system Archiver object.
 externvar id <HDF5Archiver> hdf5Archiver;
@@ -1199,6 +1403,9 @@ externvar id <LispArchiver> lispAppArchiver;
 //G: Predefined type descriptors for allocated blocks.
 externvar id <Symbol> t_ByteArray, t_LeafObject, t_PopulationObject;
 
+//G: Language tags (e.g. for use in FArguments)
+externvar id <Symbol> LanguageCOM, LanguageJS, LanguageJava, LanguageObjc;
+
 //#: Abbreviation for @selector().
 #define M(messageName) @selector (messageName)
 
@@ -1218,18 +1425,7 @@ extern id defobj_lookup_type (const char *name);
 //
 #import <defobj/types.h>
 
-
-//F: initialize defobj with application info, custom Arguments class, and
-//F: optional (or NULL) extra options w/ processing function.
-extern void initDefobj (int argc,
-                        const char **argv, 
-                        const char *appName,
-                        const char *version,
-                        const char *bugAddress,
-                        Class argumentsClass,
-                        struct argp_option *options,
-                        int (*optionFunc) (int key, const char *arg),
-                        BOOL inhibitExecutableSearchFlag);
+extern void initDefobj (id <Arguments> arguments);
 
 //F: internal module initialization function
 extern void _obj_initModule (void *module);
@@ -1303,6 +1499,8 @@ extern char *zstrdup (id <Zone> aZone, const char *str);
 
 #define SSTRDUP(str) ZSTRDUP(scratchZone, str)
 
+#define GSTRDUP(str) ZSTRDUP(globalZone, str)
+
 #define OSTRDUP(obj, str) ZSTRDUP([obj getZone], str)
 
 #define STRDUP(str) OSTRDUP(self, str)
@@ -1314,3 +1512,5 @@ extern char *zstrdup (id <Zone> aZone, const char *str);
 #define OFREEBLOCK(obj, block) ZFREEBLOCK ([obj getZone], block)
 
 #define FREEBLOCK(block) OFREEBLOCK(self, block)
+#endif
+#endif
